#ifndef PagingLandScapeFrameListener_H
#define PagingLandScapeFrameListener_H

class PagingLandScapeFrameListener : 
    public FrameListener, public KeyListener, 
    public MouseListener, public MouseMotionListener
{

    public:

         
        //---------------------------------------------------------------------
        PagingLandScapeFrameListener(RenderWindow* win, 
            Camera* cam, 
            bool useBufferedInputKeys = false, 
			bool useBufferedInputMouse = false)
			: 
            mEventProcessor (0),
            mInputDevice (0),
            mCurrentCamera (cam),
            mWindow (win),

			mShouldShutdown(false),
			mShouldRotate(false), 
			mShouldTranslate(false),

            mUseBufferedInputKeys (useBufferedInputKeys), 
            mUseBufferedInputMouse (useBufferedInputMouse),

            mAniso (1), 
            mFiltering (TFO_BILINEAR),
            mTimeUntilNextToggle (0.0f),
            mNumScreenShots (0),
            paginglandScapeevents (0),
            mCursor (0)
        {

			assert (cam);
			mCurrentCamera = cam;
			mScnMgr = cam->getSceneManager ();
			assert (mScnMgr);
			mRay.setDirection (Vector3::NEGATIVE_UNIT_Y);
            mRayQuery = mScnMgr->createRayQuery(mRay);

			mUseBufferedInputKeys = useBufferedInputKeys;
			mUseBufferedInputMouse = useBufferedInputMouse;
            mInputTypeSwitchingOn = mUseBufferedInputKeys || mUseBufferedInputMouse;
			assert (mInputTypeSwitchingOn);
            if (mInputTypeSwitchingOn)
            {
                mEventProcessor = new EventProcessor();
			    mEventProcessor->initialise(win);


			    mEventProcessor->startProcessingEvents();
                mEventProcessor->addKeyListener(this);
                mEventProcessor->addMouseListener (this);
                mEventProcessor->addMouseMotionListener (this);
			    mInputDevice = mEventProcessor->getInputReader();

                Overlay *OverCursor = OverlayManager::getSingleton().create ("myCursor");
                mCursor = new CursorOverlayElement ("myCursor");
                mCursor->initialise();
                mCursor->setMaterialName ("Cursor/default");
                mCursor->setDimensions (32.0/win->getWidth(), 32.0/win->getHeight());
                OverCursor->add2D (mCursor);
                OverCursor->show ();
                mCursor->show ();

// 				mUseBufferedInputKeys = false;
// 				mUseBufferedInputMouse = false;  
// 				mInputDevice->setBufferedInput(false, false);
// 				mInputTypeSwitchingOn = mUseBufferedInputKeys || mUseBufferedInputMouse;
			}
            else
            {
                mInputDevice = PlatformManager::getSingleton().createInputReader();
                mInputDevice->initialise(win, true, true);
            }


			mMouseMoved = true;
            mClickActive = false;
            vec = Vector3::ZERO;
            mScnMgr->setShadowTechnique (SHADOWTYPE_NONE);	
			// show FPS
			showDebugOverlay(true);
			mKeyClickedNum = 0;
			mMoveScale = 1000;
        };
        //---------------------------------------------------------------------
        ~PagingLandScapeFrameListener()
        {
            if (mInputTypeSwitchingOn)
            {
                delete mEventProcessor;
                delete mCursor;
            }
            //delete mInputDevice;
            // Destroy the query
            mScnMgr->destroyQuery (mRayQuery); 

        };
        //---------------------------------------------------------------------
        virtual void mouseClicked(MouseEvent* e) 
		{    
			if (e->getButtonID() == MouseEvent::BUTTON0_MASK)
			{
				mClickActive = true;
			}  
            e->consume();
        }
        //---------------------------------------------------------------------
        virtual void mouseEntered(MouseEvent* e) 
         {      
            e->consume();
         }
        //---------------------------------------------------------------------
        virtual void mouseExited(MouseEvent* e) 
        {      
            e->consume();
        }
        //---------------------------------------------------------------------
        virtual void mouseMoved(MouseEvent* e) 
		{ 
			const int id = e->getButtonID();

			const Real clickX = static_cast <Real> (e->getX());
			const Real clickY = static_cast <Real> (e->getY());

			mClickX = clickX;
			mClickY = clickY;

            mCursor->mouseMoved(e);

			e->consume();
        }
        //---------------------------------------------------------------------
        virtual void mouseDragged(MouseEvent* e)
        {
            
            if ((e->getButtonID() == MouseEvent::BUTTON0_MASK))
            {
                mClickX = (Real) e->getX();
                mClickY = (Real) e->getY();
                mClickActive = true;
            }
            mouseMoved (e);
            e->consume();
        };
        //---------------------------------------------------------------------
        virtual void mousePressed(MouseEvent* e)
		{
			if (e->getButtonID() == MouseEvent::BUTTON0_MASK)
			{
				mClickActive = true;
			}
			e->consume();
        };
        //---------------------------------------------------------------------
        virtual void mouseReleased(MouseEvent* e)
        {          
            mClickActive = false;          
            e->consume();
        };
        //---------------------------------------------------------------------
        // Override frameStarted event to process that (don't care about frameEnded)
	    bool frameStarted(const FrameEvent& evt)
		{
			if (mWindow->isClosed() || mShouldShutdown)
				return false;

            const Real t = evt.timeSinceLastFrame;
		    // Move about 100 units per second,
		    Real moveScale = 50.0 * t;
		    // Take about 10 seconds for full rotation
		    Real rotScale = 36 * t;
                                    

            if (mTimeUntilNextToggle >= 0.0f)
            {
                mTimeUntilNextToggle -= t;
            }

            if (paginglandScapeevents && paginglandScapeevents->hasEvent())
            {
                mWindow->setDebugText(paginglandScapeevents->getEventString());
                paginglandScapeevents->clearEventString();
            }
        
		    // Grab input device state
            if (!mInputTypeSwitchingOn)
				//(!mUseBufferedInputKeys || !mUseBufferedInputMouse)
    	    {
                mInputDevice->capture();
            }


            if (!mUseBufferedInputKeys)
			{

				if( mInputDevice->isKeyDown(KC_ESCAPE) )
				{
					return false;
				}  


		        // use CTRL key for boost
		        if (mInputDevice->isKeyDown(KC_LCONTROL))
                {
                    moveScale = moveScale * 30;
                }

                // use CTRL key for mega boost
                if (mInputDevice->isKeyDown(KC_SPACE))
		        {
			        moveScale = moveScale * 300;
		        }


                vec = Vector3::ZERO;

		        if (mInputDevice->isKeyDown(KC_A) || 
                    mInputDevice->isKeyDown(KC_Q))
		        {
			        // Move camera left
			        vec.x = -moveScale;
		        }

		        if (mInputDevice->isKeyDown(KC_D))
		        {
			        // Move camera RIGHT
			        vec.x = moveScale;
		        }

		        if (mInputDevice->isKeyDown(KC_UP) || 
                    mInputDevice->isKeyDown(KC_W)  ||
                    mInputDevice->isKeyDown(KC_Z))
		        {
			        // Move camera forward
			        vec.z = -moveScale;
		        }

		        if (mInputDevice->isKeyDown(KC_DOWN) || mInputDevice->isKeyDown(KC_S))
		        {
			        // Move camera backward
			        vec.z = moveScale;
		        }

		        if (mInputDevice->isKeyDown(KC_PGUP))
		        {
			        // Move camera up
			        vec.y = moveScale;
		        }

		        if (mInputDevice->isKeyDown(KC_PGDOWN))
		        {
			        // Move camera down
			        vec.y = -moveScale;
		        }

		        if (mInputDevice->isKeyDown(KC_RIGHT))
		        {
                    mCurrentCamera->yaw(Degree (-rotScale));
		        }
		        if (mInputDevice->isKeyDown(KC_LEFT))
		        {
			        mCurrentCamera->yaw(Degree (rotScale));
		        }

				if(mTimeUntilNextToggle < 0.0f)
				{
					const Real Keyrate = 1.5f;

					if (mInputTypeSwitchingOn && 
						(mInputDevice->isKeyDown(KC_M) || mInputDevice->isKeyDown(KC_COMMA)))
					{
						switchMouseMode();
						mTimeUntilNextToggle = Keyrate;
					}
					if (mInputDevice->isKeyDown(KC_F))
					{
						showDebugOverlay(true);
						mTimeUntilNextToggle = Keyrate;
					}

					// Toggle between WireFrame and Solid rendering
					if (mInputDevice->isKeyDown(KC_T))
					{
						if (mCurrentCamera->getPolygonMode() == PM_SOLID)
						{
							mCurrentCamera->setPolygonMode(PM_WIREFRAME);
							//showDebugOverlay(false);
							//OverlayManager::getSingleton().getByName( "PagingLandScape2/DebugOverlay" ) -> hide();
						}
						else
						{
							mCurrentCamera->setPolygonMode(PM_SOLID);
							//showDebugOverlay(true);
							//OverlayManager::getSingleton().getByName( "PagingLandScape2/DebugOverlay" ) -> show();
						}
						mTimeUntilNextToggle = Keyrate;
					}

					// Capture the screen to a PNG graphics file
					if (mInputDevice->isKeyDown(KC_SYSRQ))
					{
						char tmp[35];
						sprintf (tmp, "Demo_PagingLandScape_%d.png", ++mNumScreenShots);
						mWindow->writeContentsToFile(tmp);
						mWindow->setDebugText(String("Wrote ") + tmp);
						mTimeUntilNextToggle = Keyrate;
					}
					// Change Shadow Modes
					if (mInputDevice->isKeyDown(KC_C))
					{
						static Ogre::uint shadowtype = 0;
						shadowtype += 1;
						if (shadowtype > 3)
							shadowtype = 0;
						switch (shadowtype)
						{
						case 1:
							mScnMgr->setShadowTechnique (SHADOWTYPE_TEXTURE_MODULATIVE);	
							mScnMgr->setShadowColour(Ogre::ColourValue(0.5, 0.5, 0.5));//config
							mScnMgr->setShadowFarDistance(60);//config
							mScnMgr->setShadowTextureSettings(512, 2);//config
							//mSceneMgr->setShadowTextureSettings(ShadowSize, 2);
							break;
						case 2:
							mScnMgr->setShadowTechnique (SHADOWTYPE_STENCIL_ADDITIVE);	
							break;
						case 3:
							mScnMgr->setShadowTechnique (SHADOWTYPE_STENCIL_MODULATIVE );	
							break;
						default:
							mScnMgr->setShadowTechnique (SHADOWTYPE_NONE);	
							break;
						}
						mTimeUntilNextToggle = Keyrate;
					} 

					if (mInputDevice->isKeyDown(KC_Y))
					{
						switch(mFiltering)
						{
						case TFO_BILINEAR:
							mFiltering = TFO_TRILINEAR;
							mAniso = 1;
							break;
						case TFO_TRILINEAR:
							mFiltering = TFO_ANISOTROPIC;
							mAniso = 8;
							break;
						case TFO_ANISOTROPIC:
							mFiltering = TFO_BILINEAR;
							mAniso = 1;
							break;
						default:
							break;
						}
						MaterialManager::getSingleton().setDefaultTextureFiltering(mFiltering);
						MaterialManager::getSingleton().setDefaultAnisotropy(mAniso);
						showDebugOverlay(false);
						showDebugOverlay(true);
						mTimeUntilNextToggle = Keyrate;
					}

				}
            }

			
            if (!mUseBufferedInputMouse)
             {
          
                // Rotate view by mouse relative position
		        Radian rotX = Degree(-mInputDevice->getMouseRelativeX() * 0.13);
		        Radian rotY = Degree(-mInputDevice->getMouseRelativeY() * 0.13);

		        // Make all the changes to the camera
		        // Note that YAW direction is around a fixed axis (freelook stylee) rather than a natural YAW (e.g. airplane)
		        mCurrentCamera->yaw(rotX);
		        mCurrentCamera->pitch(rotY);

		        mCurrentCamera->moveRelative(vec);
            }
			else
			{
				mCurrentCamera->yaw(mRotX);
				mCurrentCamera->pitch(mRotY);
				mCurrentCamera->moveRelative(mTranslateVector);
			}

           updateStats();
		    // Return true to continue rendering
		    return true;
	    }
        //---------------------------------------------------------------------
        bool frameEnded(const FrameEvent& evt)
        {		   
            return true;
        };
        //---------------------------------------------------------------------
        //handles Changes in input mode
        void switchMouseMode()
	    {
            mUseBufferedInputMouse = !mUseBufferedInputMouse;
            if (mUseBufferedInputMouse)
                mCursor->show ();
            else
                mCursor->hide ();

		    mInputDevice->setBufferedInput(mUseBufferedInputKeys, mUseBufferedInputMouse);
	    }
        //---------------------------------------------------------------------
        void switchKeyMode()
	    {
            mUseBufferedInputKeys = !mUseBufferedInputKeys;
		    mInputDevice->setBufferedInput(mUseBufferedInputKeys, mUseBufferedInputMouse);
	    }
        //---------------------------------------------------------------------
        // FROM LISTENERS
	    virtual void keyClicked(KeyEvent* e)
		{
			mTranslateVector = Vector3::ZERO;
			mKeyClickedNum += 1;
            
	    }
        //---------------------------------------------------------------------
		virtual void keyPressed(KeyEvent* e) 
		{
			// eScape ??

			if (e->getKey() == KC_M || 
				e->getKey() == KC_COMMA)
			{
				switchMouseMode();
			}
			else if (e->getKey() == KC_J)
			{
				switchKeyMode();
			}

			if (e->getKey() == KC_ESCAPE)
			{
				mShouldShutdown = true;
			}

			if((e->getKey() == KC_A)
				|| (e->getKey() == KC_Q))
			{
				// Move camera left
				mTranslateVector.x = -mMoveScale;
			}

			if ((e->getKey() == KC_D))
			{
				// Move camera RIGHT
				mTranslateVector.x = mMoveScale;
			}

			/* Move camera forward by key press. */
			if ((e->getKey() == KC_UP)
				|| (e->getKey() == KC_W) 
				|| (e->getKey() == KC_Z))
			{
				mTranslateVector.z = -mMoveScale;
			}

			/* Move camera backward by key press. */
			if ((e->getKey() == KC_DOWN)
				|| (e->getKey() == KC_S))
			{
				mTranslateVector.z = mMoveScale;
			}

			if ((e->getKey() == KC_PGUP))
			{
				// Move camera up
				mTranslateVector.y = mMoveScale;
			}

			if ((e->getKey() == KC_PGDOWN))
			{
				// Move camera down
				mTranslateVector.y = -mMoveScale;
			}

			if ((e->getKey() == KC_RIGHT))
			{
				mCurrentCamera->yaw(-mRotScale);
			}

			if ((e->getKey() == KC_LEFT))
			{
				mCurrentCamera->yaw(mRotScale);
			}

		}
        //---------------------------------------------------------------------
		virtual void keyReleased(KeyEvent* e) 
		{ 
			mKeyClickedNum -= 1;
			if (mKeyClickedNum == 0)
				mTranslateVector = Vector3::ZERO;
		}
    

    protected:

        //---------------------------------------------------------------------
        void showDebugOverlay(bool show)
        {
            Overlay* o = (Overlay*)OverlayManager::getSingleton().getByName("Core/DebugOverlay");
            if (!o)
                OGRE_EXCEPT( Exception::ERR_ITEM_NOT_FOUND, "Could not find overlay Core/DebugOverlay",
                    "showDebugOverlay" );
            if (o->isVisible() && show)
			{
				o->hide();
            }
            else if (show)
			{
				o->show();
            }
        };
        //---------------------------------------------------------------------
        void updateStats(void)
        {
            static String currFps = "Current FPS: ";
            static String avgFps = "Average FPS: ";
            static String bestFps = "Best FPS: ";
            static String worstFps = "Worst FPS: ";
            static String tris = "Triangle Count: ";

            // update stats when necessary
            OverlayElement* guiAvg = OverlayManager::getSingleton().getOverlayElement("Core/AverageFps");
            OverlayElement* guiCurr = OverlayManager::getSingleton().getOverlayElement("Core/CurrFps");
            OverlayElement* guiBest = OverlayManager::getSingleton().getOverlayElement("Core/BestFps");
            OverlayElement* guiWorst = OverlayManager::getSingleton().getOverlayElement("Core/WorstFps");

            const RenderTarget::FrameStats& stats = mWindow->getStatistics();

            guiAvg->setCaption(avgFps + StringConverter::toString(stats.avgFPS));
            guiCurr->setCaption(currFps + StringConverter::toString(stats.lastFPS));
            guiBest->setCaption(bestFps + StringConverter::toString(stats.bestFPS)
                +" "+StringConverter::toString(stats.bestFrameTime)+" ms");
            guiWorst->setCaption(worstFps + StringConverter::toString(stats.worstFPS)
                +" "+StringConverter::toString(stats.worstFrameTime)+" ms");

            OverlayElement* guiTris = OverlayManager::getSingleton().getOverlayElement("Core/NumTris");
            guiTris->setCaption(tris + StringConverter::toString((Ogre::uint)stats.triangleCount));

            OverlayElement* guiDbg = OverlayManager::getSingleton().getOverlayElement("Core/DebugText");
            guiDbg->setCaption(mWindow->getDebugText());
        };

		//---------------------------------------------------------------------
		Vector3 getHeight(const Vector3 &pos, const Real heightAbove)
		{
			// A RayQuery with the direction set to Vector3::UNIT_Y will always return
			// one and only one result.  Distance will always be 0 for a UNIT_Y 
			// and you can access the intersection point via worldFragment->singleIntersection
			mRay.setOrigin (pos);
			mRayQuery->setRay (mRay); 
			RaySceneQueryResult& qryResult = mRayQuery->execute();
			RaySceneQueryResult::iterator it = qryResult.begin();
			if (it != qryResult.end() && it->worldFragment)
			{
				const Real height =  it->worldFragment->singleIntersection.y;
				return Vector3(pos.x, heightAbove + heightAbove, pos.z);
			}
			return Vector3(pos.x, heightAbove, pos.z);
		}

		//---------------------------------------------------------------------
		bool getIntersection(Vector3 &intersectPoint, const Real ClickX, const Real clickY)
		{
			Ray pickRay = mCurrentCamera->getCameraToViewportRay(ClickX, clickY);

			mRayQuery->setRay (pickRay);
			//mRayQuery->setSortByDistance (true, 1); not useful on only one result shot
			mRayQuery->setQueryMask (RSQ_FirstTerrain); //PLSM2 only
			mRayQuery->execute ();
			RaySceneQueryResult& qryResult = mRayQuery->execute();
			RaySceneQueryResult::iterator it = qryResult.begin();
			if (it != qryResult.end() && it->worldFragment)
			{
				intersectPoint = it->worldFragment->singleIntersection;
				return true;
			}
			return false;
		}

    protected:
        EventProcessor* mEventProcessor;
		InputReader* mInputDevice;
		Camera* mCurrentCamera;
        SceneManager *mScnMgr;
        RenderWindow* mWindow;

		bool mMouseMoved;
        bool mUseBufferedInputKeys, mUseBufferedInputMouse, mInputTypeSwitchingOn;

        unsigned int mNumScreenShots;

        // just to stop toggles flipping too fast
        Real mTimeUntilNextToggle;

        TextureFilterOptions mFiltering;
        int mAniso;

		bool                 mShouldRotate; 
		bool                 mShouldTranslate;
		bool                 mShouldShutdown;

		Vector3              mTranslateVector;
		unsigned int         mKeyClickedNum;

        RaySceneQuery* mRayQuery;
		Ray mRay;
        bool mClickActive;
        Real mClickX, mClickY;

		Real mMoveScale;

		Radian mRotScale;
		Radian mRotX;
		Radian mRotY;

		Vector3 vec;
		
		
        CustomPagingLandScapeListener *paginglandScapeevents;
        CursorOverlayElement *mCursor;
};

#endif
